using System;
using System.Drawing;
using System.Windows.Forms;

namespace Photo_Editor_Application
{
    /// <summary>
    /// Picture class stores all image related data.
    /// It also provides the basic functionality.
    /// </summary>
	public class Picture
	{
        // private fields used internally
        private Bitmap loadedImage;
        private OpenFileDialog loadFileDialog;
        private SaveFileDialog saveFileDialog;

        /// <summary>
        /// Picture Class contructor.
        /// The image class stores the image and is
        /// responsible for basic image operations.
        /// </summary>
		public Picture()
		{
            string defaultImage = PhotoEditorForm.GetApplicationDirectory + @"\Hawaii.jpg";
            
            loadedImage = new Bitmap(defaultImage);
            loadFileDialog = new OpenFileDialog();
            saveFileDialog = new SaveFileDialog();

		}

        /// <summary>
        /// Opens Open File Dialog and loads the selected image.
        /// </summary>
        /// <requirement>F:image_load_and_save</requirement>
        public void LoadImage()
        {
            loadFileDialog.Filter = " jpg files (*.jpg)|*.jpg| gif files (*.gif)|*.gif| bmp files (*.bmp)|*.bmp| All files (*.*)|*.*";
            loadFileDialog.ShowDialog();
            loadedImage = new Bitmap(loadFileDialog.FileName);
        }
    
        /// <summary>
        /// Method called from CropDialog. The current
        /// shown image is cropped to the size provided
        /// in the parameters. The cropping is done
        /// with a rectangle whose center is put on
        /// the center of the image.
        /// </summary>
        /// <requirement>F:image_crop</requirement>
        /// <param name="newHeight">height of the cropped image</param>
        /// <param name="newWidth">width of the cropped image</param>
        public void CropImage(int newWidth, int newHeight)
        {
            // Check that corpping region is
            // actually within the image and that
            // the values provided are positive
            if((newWidth < loadedImage.Size.Width) && 
                (newHeight < loadedImage.Size.Height) &&
                (newHeight > 0) && (newWidth > 0))
            {
                int xmin = (loadedImage.Size.Width / 2) - 
                    (newWidth / 2);
                int xdim = newWidth;
                int ymin = (loadedImage.Size.Height / 2) - 
                    (newHeight / 2);
                int ydim =	newHeight;
                Rectangle rectangle = new Rectangle(xmin, ymin, 
                    xdim, ydim);
                if(rectangle.IsEmpty)
                {
                    throw(new Exception("Error, CropImage failed to allocate clipping rectangle"));
                }
                Bitmap croppedImage = loadedImage.Clone(rectangle,
                    System.Drawing.Imaging.PixelFormat.DontCare);
                Bitmap oldImage = loadedImage;
                loadedImage = new Bitmap(croppedImage, 
                    rectangle.Size);
                if(loadedImage == null)
                {
                    throw(new Exception("Error, Image memory allocation failed"));
                }
            }
        }
        /// <summary>
        /// Flips and/or rotates the Image
        /// </summary>
        /// <requirement>F:image_flip</requirement>
        /// <requirement>F:image_rotate</requirement>
        /// <param name="rotateFlip">Enum type specifying the user requested rotation or flip operation</param>
        public void RotateFlipImage(RotateFlipType rotateFlip)
        {
            loadedImage.RotateFlip(rotateFlip);
            
        }
        /// <summary>
        /// Saves the current image
        /// used as test hook.
        /// </summary>
        public void SaveImage()
        {
            saveFileDialog.Filter = " jpg files (*.jpg)|*.jpg|gif files (*.gif)|*.gif| bmp files (*.bmp)|*.bmp|All files (*.*)|*.*";
            saveFileDialog.ShowDialog();
            loadedImage.Save(saveFileDialog.FileName);
            saveFileDialog.Dispose();
        }

        /// <summary>
        /// Accessor to get the currently loaded image.
        /// </summary>
        public Bitmap LoadedImage
        {
            get
            {
                return loadedImage;
            }
            set
            {
                
            }
        }

        /// <summary>
        /// Opens the file with the provided Name.
        /// </summary>
        /// <param name="fileName">name of the file to be opened</param>
        /// <requirement>F:image_load_and_save</requirement>
        public void LoadImage(string fileName)
        {	
            Bitmap oldImage;
            oldImage = loadedImage;
            loadedImage = new Bitmap(fileName);
            if(loadedImage == null)
            {
                throw(new Exception("Error, LoadImage with filename failed"));
            }
            oldImage.Dispose();

        }
        /// <summary>
        /// Saves the current image
        /// with the provided fileName.
        /// </summary>
        /// <param name="fileName">Name under which the image is saved</param>
        public void SaveImage(string fileName)
        {
            loadedImage.Save(fileName);
        }


       
	}
}
